#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt4.QtCore import QByteArray, QString, QThread, QMutex, pyqtSignal, pyqtSlot
from PyQt4.QtNetwork import QTcpSocket, QHostAddress, QAbstractSocket, QNetworkProxy

SocketErrors = {
	0 : 'ConnectionRefusedError\nThe connection was refused by the peer (or timed out).', \
	1 : 'RemoteHostClosedError\nThe remote host closed the connection. Note that the client socket (i.e., this socket) will be closed after the remote close notification has been sent.', \
	2 : 'HostNotFoundError\nThe host address was not found.', \
	3 : 'SocketAccessError\nThe socket operation failed because the application lacked the required privileges.', \
	4 : 'SocketResourceError\nThe local system ran out of resources (e.g., too many sockets).', \
	5 : 'SocketTimeoutError\nThe socket operation timed out.', \
	6 : 'DatagramTooLargeError\nThe datagram was larger than the operating system`s limit (which can be as low as 8192 bytes).', \
	7 : 'NetworkError\nAn error occurred with the network (e.g., the network cable was accidentally plugged out).', \
	8 : 'AddressInUseError\nThe address specified to QTcpSocet.bind() is already in use and was set to be exclusive.', \
	9 : 'SocketAddressNotAvailableError\nThe address specified to QTcpSocet.bind() does not belong to the host.', \
	10 : 'UnsupportedSocketOperationError\nThe requested socket operation is not supported by the local operating system (e.g., lack of IPv6 support).', \
	12 : 'ProxyAuthenticationRequiredError\nThe socket is using a proxy, and the proxy requires authentication.', \
	13 : 'SslHandshakeFailedError\nThe SSL/TLS handshake failed, so the connection was closed (only used in QSslSocket)', \
	11 : 'UnfinishedSocketOperationError\nUsed by QAbstractSocketEngine only, The last operation attempted has not finished yet (still in progress in the background).', \
	14 : 'ProxyConnectionRefusedError\nCould not contact the proxy server because the connection to that server was denied', \
	15 : 'ProxyConnectionClosedError\nThe connection to the proxy server was closed unexpectedly (before the connection to the final peer was established)', \
	16 : 'ProxyConnectionTimeoutError\nThe connection to the proxy server timed out or the proxy server stopped responding in the authentication phase.', \
	17 : 'ProxyNotFoundError\nThe proxy address set with setProxy() (or the application proxy) was not found.', \
	18 : 'ProxyProtocolError\nThe connection negotiation with the proxy server because the response from the proxy server could not be understood.', \
	-1 : 'UnknownSocketError\nAn unidentified error occurred.'
}

class TcpClient(QThread):
	def __init__(self, parent, addr = None):
		QThread.__init__(self, parent)

		self.prnt = parent
		self.addr = QHostAddress(QHostAddress.LocalHost) if addr is None else QHostAddress(addr)
		proxy = QNetworkProxy(QNetworkProxy.Socks5Proxy, 'localhost', 9050)
		proxy.setCapabilities(QNetworkProxy.TunnelingCapability)
		self.tcp = QTcpSocket()
		self.tcp.setProxy(proxy)
		print 'bind to tcp:', self.addr.toString()
		self.tcp.connectToHost(self.addr, 34001)
		self.tcp.error.connect(self.errorAnalyser)
		#self.tcp.readyRead.connect(self.readTcp)
		print "Binding..."
		self.STOP = False
		self.locker = QMutex()

	def run(self):
		self.prnt.initAvahiService()
		while True :
			if self.tcp is not None and self.tcp.state() == QAbstractSocket.ConnectedState :
				if self.tcp.waitForReadyRead() : self.readTcp()
			else : self.msleep(100)
			if self.STOP and self.tcp is not None: self.tcp.close(); break
		print 'TCPClient closed...'
		self.prnt.changeConnectState.emit()

	def stop(self):
		self.locker.lock()
		self.STOP = True
		self.locker.unlock()

	def readTcp(self):
			try :
				datagram = self.tcp.readData()
				print "TcpDatagram: [%s]" % (QString().fromUtf8(data))
				self.prnt.contactMessage.emit(QString().fromUtf8(data), self.addr.toString())
			except socket.error, err :
				print '[in readTcp() TcpClient] SocketError1 : ', err
			except socket.timeout, err :
				print '[in readTcp() TcpClient] SocketError2 : ', err
			except :
				print '[in readTcp() TcpClient] UnknownError'
			finally : pass

	def errorAnalyser(self, error):
		print '[TcpClient] SocketError : ', SocketErrors[error]

if __name__ == '__main__':
	tcp = TcpClient()
	try :
		tcp.start()
	except KeyboardInterrupt :
		print 'tread closed manually'
		tcp.stop()

